/* gcc `pkg-config --cflags --libs x11` -o rectangles rectangles.c */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>

#define MEASUREMENTS 5
#define ITERATIONS 4000

int
main (int argc, char **argv)
{
    Display *dpy;
    Window win;
    XSetWindowAttributes attr;
    XEvent xev;
    GC gc, gc_white, gc_black;
    unsigned long gcmask = 0l;
    XGCValues gcvalues;
    struct timeval tv_start, tv_stop;
    int i, j;
    double elapsed;
    XImage *ximage;

    dpy = XOpenDisplay (NULL);
    if (dpy == NULL) {
	fprintf (stderr, "Failed to open display %s\n", XDisplayName (NULL));
	exit (1);
    }

    attr.override_redirect = True;
    win = XCreateWindow (dpy, DefaultRootWindow (dpy), 0, 0,
			 600, 600, 0, DefaultDepth (dpy, DefaultScreen (dpy)),
			 InputOutput, DefaultVisual (dpy, DefaultScreen (dpy)),
			 CWOverrideRedirect, &attr);

    gcmask |= GCFunction;
    gcvalues.function = GXcopy;
    gcmask |= GCForeground;
    gcvalues.foreground = WhitePixel (dpy, DefaultScreen (dpy));
    gc_white = XCreateGC (dpy, win, gcmask, &gcvalues);

    gcvalues.foreground = BlackPixel (dpy, DefaultScreen (dpy));
    gc_black = XCreateGC (dpy, win, gcmask, &gcvalues);

    XMapWindow (dpy, win);

    gc = gc_black;
    for (i = 0; i < MEASUREMENTS; i++) {
	/* Nothing fancy here, just a gettimeofday system call. */
	gettimeofday (&tv_start, NULL);
	for (j = 0; j < ITERATIONS; j++) {
	    XFillRectangle (dpy, win, gc, 0, 0, 500, 500);
	    if (gc == gc_white)
		gc = gc_black;
	    else
		gc = gc_white;
	}
	/* Standard 1x1 XGetImage to wait for rendering to complete. */
	ximage = XGetImage (dpy, win, 0, 0, 1, 1, AllPlanes, ZPixmap);
	gettimeofday (&tv_stop, NULL);
	XDestroyImage (ximage);

	elapsed = tv_stop.tv_sec - tv_start.tv_sec +
	    (tv_stop.tv_usec - tv_start.tv_usec) / 1000000.0;
	printf ("%d iterations @ %.4f msec ( %.1f/sec): 500x500 XFillRectangle\n",
		ITERATIONS, elapsed * 1000 / ITERATIONS, ITERATIONS / elapsed);
    }
	    
    XCloseDisplay (dpy);

    return 0;
}
